#!/usr/bin/python3
import argparse
import numpy as np
import os
import sys

def synchronize(video_file, subtitle_file, output_file, verbose=False, model_file=None, **kwargs):
    """
    Automatically synchronize subtitles with audio in a video file.
    Uses FFMPEG to extract the audio from the video file and the command line
    tool "ffmpeg" must be available. Uses temporary files which are deleted
    automatically.

    :param video_file: Input video file name (string)
    :param subtitle_file: Input SRT subtitle file name (string)
    :param output_file: Output (syncrhonized) SRT subtitle file name (string)
    :param verbose: If True, print progress information to stdout
    :param kwargs: Search parameters, see ``autosubsync --help``
    :return: True on success (quality of fit test passed), False if failed.
    """

    # these are here to enable running as python3 autosubsync/main.py
    from autosubsync import model
    from autosubsync import predict
    from autosubsync import preprocessing
    from autosubsync.quality_of_fit import threshold

    if model_file is None:
        from pkg_resources import resource_filename
        model_file = resource_filename(__name__, '../trained-model.bin')

    trained_model = model.load(model_file)

    if verbose: print('Extracting audio using ffmpeg and reading subtitles...')
    target_data = preprocessing.import_target_files(video_file, subtitle_file)

    if verbose: print('audio and subtitles extracted, fitting...')
    transform_func, quality = predict.main(trained_model, *target_data, verbose=verbose, **kwargs)

    success = quality > threshold
    if verbose:
        print('quality of fit: %g, threshold %g' % (quality, threshold))
        print('Fit complete. Performing resync, writing to ' + output_file)

    preprocessing.transform_srt(subtitle_file, output_file, transform_func)

    if verbose and success:
        print('success!')

    return success

def cli(packaged_model=False):
    p = argparse.ArgumentParser(description=synchronize.__doc__.split('\n\n')[0])
    p.add_argument('video_file', help='Input video file')
    p.add_argument('subtitle_file', help='Input SRT subtitle file')
    p.add_argument('output_file', help='Output (auto-syncrhonized) SRT subtitle file')

    # Make model file an argument only in the non-packaged version
    if not packaged_model:
        p.add_argument('--model_file', default='trained-model.bin')

    p.add_argument('--max_shift_secs', default=20.0, type=float,
        help='Maximum subtitle shift in seconds (default 20)')
    p.add_argument('--parallelism', default=3, type=int,
        help='Number of parallel worker processes (default 3)')
    p.add_argument('--fixed_skew', default=None,
        help='Use a fixed skew (e.g. 1) instead of auto-detection')
    p.add_argument('--silent', action='store_true',
        help='Do not print progress information')
    args = p.parse_args()

    if packaged_model:
        model_file = None
    else:
        model_file = args.model_file

    success = synchronize(args.video_file, args.subtitle_file, args.output_file,\
                  verbose = not args.silent,
                  model_file = model_file,
                  max_shift_secs = args.max_shift_secs, \
                  parallelism = args.parallelism,
                  fixed_skew = args.fixed_skew)

    if not success:
        sys.stderr.write("\nWARNING: low quality of fit. Wrong subtitle file?\n")
        sys.exit(1)

def cli_packaged():
    """Entry point in the packaged, pip-installable version"""
    cli(packaged_model=True)

if __name__ == '__main__':
    # Entry point for running from repository root folder
    sys.path.append('.')
    cli(packaged_model=False)
